%--------------------------------------------------------------------------------
% Project       : SWI-Speech
% File          : SWI_Speech.pl
% Purpose       : Prolog Interface to SAPI speech recognition and synthesis engine.
% System        : SWI-Prolog 5.0.8, MS SAPI 5.0
% Author        : Jonathan T. McClain
% Institution   : The University of Georgia Artificial Intelligence Center
% Last Modified : 5/8/03
%--------------------------------------------------------------------------------

%--------------------------------------------------------------------------------
:- dynamic(foreign_library_loaded/1).
:- dynamic(attribute/3).
:- dynamic(volume/1).
:- dynamic(rate/1).
%--------------------------------------------------------------------------------

attribute(gender,"Gender=Male",required).
attribute(age,"Age=Adult",required).
volume(100).
rate(0).

%--------------------------------------------------------------------------------
% ensure_foreign_library_loaded
%  Ensures that swi_speech.dll is loaded into the system. swi_speech.dll must be
%  loaded before any calls may be made to the predicates it defines.
%--------------------------------------------------------------------------------
ensure_foreign_library_loaded :- ensure_foreign_library_loaded(swi_speech).

ensure_foreign_library_loaded(ForeignLibrary) :-
  foreign_library_loaded(ForeignLibrary),
  !.

ensure_foreign_library_loaded(ForeignLibrary) :-
  load_foreign_library(ForeignLibrary),
  assert(foreign_library_loaded(ForeignLibrary)).

%--------------------------------------------------------------------------------
% listen(-Sentence)
%  Uses SAPI's speech recognition engine to listen for voice input. Voice input
%  is returned in the form of a quoted atom (eg. 'hello'), which can be easily
%  changed to a character string using atom_chars/2 for processing.
%--------------------------------------------------------------------------------
listen(Sentence) :-
   ensure_foreign_library_loaded,
   swi_listen(Sentence).

%--------------------------------------------------------------------------------
% speak(+Sentence)
%  Uses SAPI's speech sysnthesis engine to speak a line of text. This method
%  uses whatever the current voice settings are as set by the user. Input to this
%  method can be in the form of a list of characters, or a quoted atom.
%--------------------------------------------------------------------------------
speak(Sentence) :-
   ensure_foreign_library_loaded,
   swi_speak(Sentence).

%--------------------------------------------------------------------------------
% speak_with_attributes(+Sentence)
%  Uses SAPI's speech synthesis engine to speak a line of text. This method
%  uses the voice attributes that are set by the following predicates,
%  set_gender/2, set_age/2, set_rate/1, and set_volume/1. The gender and age
%  attributes are used in the following manner. The windows registry is searched
%  for voices that match the settings of gender and age. Of all the voices that
%  match, the best match is used to speak the line of text. Attributes marked as
%  required must be present in any voice selected by the search. Optional
%  attributes merely influence which voice is chosen as the best match. Input to
%  this method can be in the form of a list of characters, or a quoted atom.
%--------------------------------------------------------------------------------
speak_with_attributes(Sentence) :-
   ensure_foreign_library_loaded,
   volume(Volume),
   rate(Rate),
   get_required_list(RequiredList),
   get_optional_list(OptionalList),
   swi_speak(Sentence,Rate,Volume,RequiredList,OptionalList).

get_required_list([]) :-
   \+ attribute(_,_,required),
   !.

get_required_list(RequiredList) :-
   findall(Attribute,
	   (
	     attribute(_,Attribute,required)
	   ),
	  [FirstAttribute|Rest]),
   append_list(FirstAttribute,Rest,RequiredList).

get_optional_list([]) :-
   \+ attribute(_,_,optional),
   !.

get_optional_list(OptionalList) :-
   findall(Attribute,
	   (
	     attribute(_,Attribute,optional)
	   ),
	  [FirstAttribute|Rest]),
   append_list(FirstAttribute,Rest,OptionalList).

append_list(CompletedList,[],CompletedList).

append_list(PartialList,[List|RestOfLists],CompletedList) :-
   append(PartialList,";",Temp),
   append(Temp,List,NewPartialList),
   append_list(NewPartialList,RestOfLists,CompletedList).

%--------------------------------------------------------------------------------
% set_gender(+Gender,+Condition)
%  This method sets the gender attribute for use with speak_with_attributes/1.
%  Gender can be set to either male or female. The condition can be either set to
%  required or optional depending on whether the gender is required or just
%  preferred. The default setting for gender is male and required.
%--------------------------------------------------------------------------------
set_gender(male,optional) :-
   retract(attribute(gender,_,_)),
   assert(attribute(gender,"Gender=Male",optional)).

set_gender(male,required) :-
   retract(attribute(gender,_,_)),
   assert(attribute(gender,"Gender=Male",required)).

set_gender(female,optional) :-
   retract(attribute(gender,_,_)),
   assert(attribute(gender,"Gender=Female",optional)).

set_gender(female,required) :-
   retract(attribute(gender,_,_)),
   assert(attribute(gender,"Gender=Female",required)).

%--------------------------------------------------------------------------------
% set_age(+Age,+Condition)
%  This method sets the age attribute for use with speak_with_attributes/1.
%  Age can be set to the following values child, teen, adult, and senior. The
%  condition can be either set to required or optional depending on whether the
%  age is required or just preferred. The default setting for age is adult and
%  required.
%--------------------------------------------------------------------------------
set_age(child,optional) :-
   retract(attribute(age,_,_)),
   assert(attribute(age,"Age=Child",optional)).

set_age(child,required) :-
   retract(attribute(age,_,_)),
   assert(attribute(age,"Age=Child",required)).

set_age(teen,optional) :-
   retract(attribute(age,_,_)),
   assert(attribute(age,"Age=Teen",optional)).

set_age(teen,required) :-
   retract(attribute(age,_,_)),
   assert(attribute(age,"Age=Teen",required)).

set_age(adult,optional) :-
   retract(attribute(age,_,_)),
   assert(attribute(age,"Age=Adult",optional)).

set_age(adult,required) :-
   retract(attribute(age,_,_)),
   assert(attribute(age,"Age=Adult",required)).

set_age(senior,optional) :-
   retract(attribute(age,_,_)),
   assert(attribute(age,"Age=Senior",optional)).

set_age(senior,required) :-
   retract(attribute(age,_,_)),
   assert(attribute(age,"Age=Senior",required)).

%--------------------------------------------------------------------------------
% set_rate(+Rate)
%  This method sets the rate of speech for use with speak_with_attributes/1.
%  The rate value must be a number between -10 and 10, with -10 being the slowest
%  and 10 being the fastest. The default setting for rate is 0.
%--------------------------------------------------------------------------------
set_rate(Rate) :-
   retract(rate(_)),
   assert(rate(Rate)).

%--------------------------------------------------------------------------------
% set_volume(+Volume)
%  This method sets the volume of speech for use with speak_with_attributes/1.
%  The volume value must be a number between 0 and 100, with 0 being the lowest
%  and 100 being the highest. The default setting for volume is 100.
%--------------------------------------------------------------------------------
set_volume(Volume) :-
   retract(volume(_)),
   assert(volume(Volume)).
